@@ -136,6 +136,11 @@ class PolymorphicQuerySet(QuerySet):
         return new
 
     def as_manager(cls):
+        """
+        Override base :meth:`~django.db.models.query.QuerySet.as_manager` to return
+        a manager extended from :class:`polymorphic.managers.PolymorphicManager`.
+        """
+
         from .managers import PolymorphicManager
 
         manager = PolymorphicManager.from_queryset(cls)()
@@ -261,7 +266,9 @@ class PolymorphicQuerySet(QuerySet):
             # The field on which the aggregate operates is
             # stored inside a complex query expression.
             if isinstance(a, Q):
-                translate_polymorphic_Q_object(self.model, a)
+                # modify in place - this should be fixed if we want immutable
+                # aggregate/annotation expressions
+                a.children = translate_polymorphic_Q_object(self.model, a).children
             elif isinstance(a, FilteredRelation):
                 patch_lookup(a.condition)
             elif hasattr(a, "get_source_expressions"):
