@@ -4,6 +4,7 @@ import django
 from django.contrib.auth.models import Group
 from django.contrib.auth import get_user_model
 from django.contrib.contenttypes.models import ContentType
+from django.db.models import Manager
 from django.db import models
 from django.db.models.query import QuerySet
 
@@ -38,7 +39,7 @@ class Model2B(Model2A):
 
 
 class Model2C(Model2B):
-    field3 = models.CharField(max_length=30)
+    field3 = models.CharField(max_length=30, blank=True, default="")
 
 
 class Model2D(Model2C):
@@ -341,6 +342,18 @@ class UUIDResearchProject(UUIDProject):
     supervisor = models.CharField(max_length=30)
 
 
+class UUIDArtProjectA(UUIDArtProject): ...
+
+
+class UUIDArtProjectB(UUIDArtProjectA): ...
+
+
+class UUIDArtProjectC(UUIDArtProjectB): ...
+
+
+class UUIDArtProjectD(UUIDArtProjectC): ...
+
+
 class UUIDPlainA(models.Model):
     uuid_primary_key = models.UUIDField(primary_key=True, default=uuid.uuid1)
     field1 = models.CharField(max_length=30)
@@ -612,3 +625,148 @@ class Bar(PolymorphicMixin, PolymorphicModel):
 
 class Baz(ModelMixin, PolymorphicModel):
     pass
+
+
+class MyBaseQuerySet(PolymorphicQuerySet):
+    def filter_by_user(self, _):
+        return self.all()
+
+
+class MyBaseModel(PolymorphicModel):
+    ...
+    objects = MyBaseQuerySet.as_manager()
+
+
+class MyChild1QuerySet(MyBaseQuerySet):
+    def filter_by_user(self, num):
+        return self.filter(fieldA__lt=num)
+
+
+class MyChild1Model(MyBaseModel):
+    fieldA = models.IntegerField()
+    ...
+    objects = MyChild1QuerySet.as_manager()
+
+
+class MyChild2QuerySet(MyBaseQuerySet):
+    def filter_by_user(self, num):
+        return self.filter(fieldB__gt=num)
+
+
+class MyChild2Model(MyBaseModel):
+    fieldB = models.IntegerField()
+    ...
+    objects = PolymorphicManager.from_queryset(MyChild2QuerySet)()
+    base_manager = MyBaseQuerySet.as_manager()
+
+
+class SpecialQuerySet(PolymorphicQuerySet):
+    def has_text(self, text):
+        return self.filter(abstract_field__icontains=text)
+
+
+class SpecialPolymorphicManager(PolymorphicManager.from_queryset(SpecialQuerySet)):
+    def custom_queryset(self):
+        return self.get_queryset()
+
+
+class AbstractManagerTest(PolymorphicModel):
+    """
+    Tests that custom manager patterns work on abstract base models
+    """
+
+    objects = SpecialPolymorphicManager()
+    basic_manager = Manager()
+    default_manager = PolymorphicManager()
+
+    abstract_field = models.CharField(max_length=32)
+
+    class Meta:
+        abstract = True
+
+
+class RelatedManagerTest(models.Model): ...
+
+
+class DerivedManagerTest(AbstractManagerTest):
+    related_test = models.ForeignKey(
+        RelatedManagerTest,
+        on_delete=models.CASCADE,
+        default=None,
+        null=True,
+        related_name="derived",
+    )
+
+
+class DerivedManagerTest2(DerivedManagerTest):
+    objects = PolymorphicManager()
+
+
+class FKTestBase(PolymorphicModel): ...
+
+
+class FKTestChild(Base): ...
+
+
+class FKTest(models.Model):
+    fk = models.ForeignKey(Base, null=True, on_delete=models.SET_NULL)
+
+
+class NoChildren(PolymorphicModel):
+    field1 = models.CharField(max_length=12)
+
+
+class NormalBase(models.Model):
+    nb_field = models.IntegerField()
+
+
+class NormalExtension(NormalBase):
+    ne_field = models.CharField(max_length=12)
+
+
+class PolyExtension(PolymorphicModel, NormalExtension):
+    poly_ext_field = models.IntegerField()
+
+
+class PolyExtChild(PolyExtension):
+    poly_child_field = models.CharField(max_length=12)
+
+
+class DeepCopyTester(PolymorphicModel):
+    binary_field = models.BinaryField()
+
+
+class DeepCopyTester2(DeepCopyTester):
+    binary_field2 = models.BinaryField()
+
+
+class DucksLake(models.Model):
+    lake = models.ForeignKey("LakeWithThrough", on_delete=models.CASCADE)
+    duck = models.ForeignKey(Duck, on_delete=models.CASCADE)
+    time = models.CharField(max_length=10)
+
+
+class Lake(models.Model):
+    ducks = models.ManyToManyField(Duck)
+
+
+class LakeWithThrough(models.Model):
+    ducks = models.ManyToManyField(Duck, through=DucksLake)
+
+
+class ChoiceBlank(PolymorphicModel):
+    pass
+
+
+class ChoiceAthlete(ChoiceBlank):
+    choice = models.CharField(max_length=100)
+
+
+class BetMultiple(models.Model):
+    answer = models.ManyToManyField("ChoiceBlank", blank=True, through="RankedAthlete")
+
+
+class RankedAthlete(models.Model):
+    choiceAthlete = models.ForeignKey("ChoiceBlank", on_delete=models.CASCADE)
+    bet = models.ForeignKey("BetMultiple", on_delete=models.CASCADE)
+    rank = models.IntegerField()
