@@ -1,5 +1,4 @@
 import pytest
-import re
 import uuid
 
 from django.contrib.auth import get_user_model
@@ -25,6 +24,7 @@ from polymorphic.tests.models import (
     CustomPkBase,
     CustomPkInherit,
     Enhance_Base,
+    Enhance_Plain,
     Enhance_Inherit,
     InlineParent,
     InlineModelA,
@@ -88,6 +88,10 @@ from polymorphic.tests.models import (
     SubclassSelectorProxyConcreteModel,
     ParentLinkAndRelatedName,
     UUIDArtProject,
+    UUIDArtProjectA,
+    UUIDArtProjectB,
+    UUIDArtProjectC,
+    UUIDArtProjectD,
     UUIDPlainA,
     UUIDPlainB,
     UUIDPlainC,
@@ -175,24 +179,34 @@ class PolymorphicTests(TransactionTestCase):
         """
         object retrieval problem occuring with some custom primary key fields (UUIDField as test case)
         """
-        UUIDProject.objects.create(topic="John's gathering")
-        UUIDArtProject.objects.create(topic="Sculpting with Tim", artist="T. Turner")
-        UUIDResearchProject.objects.create(topic="Swallow Aerodynamics", supervisor="Dr. Winter")
+        up1 = UUIDProject.objects.create(topic="John's gathering")
+        up2 = UUIDArtProject.objects.create(topic="Sculpting with Tim", artist="T. Turner")
+        up3 = UUIDResearchProject.objects.create(
+            topic="Swallow Aerodynamics", supervisor="Dr. Winter"
+        )
+
+        up4 = UUIDArtProjectA.objects.create(topic="ProjectA", artist="Artist A")
+        up5 = UUIDArtProjectB.objects.create(topic="ProjectB", artist="Artist B")
+        up6 = UUIDArtProjectC.objects.create(topic="ProjectC", artist="Artist C")
+        up7 = UUIDArtProjectD.objects.create(topic="ProjectD", artist="Artist D")
 
         qs = UUIDProject.objects.all()
         ol = list(qs)
         a = qs[0]
         b = qs[1]
         c = qs[2]
-        assert len(qs) == 3
+        assert len(qs) == 7
         assert isinstance(a.uuid_primary_key, uuid.UUID)
         assert isinstance(a.pk, uuid.UUID)
 
-        res = re.sub(' "(.*?)..", topic', ", topic", repr(qs))
-        res_exp = """[ <UUIDProject: uuid_primary_key (UUIDField/pk), topic (CharField) "John's gathering">,
-  <UUIDArtProject: uuid_primary_key (UUIDField/pk), topic (CharField) "Sculpting with Tim", artist (CharField) "T. Turner">,
-  <UUIDResearchProject: uuid_primary_key (UUIDField/pk), topic (CharField) "Swallow Aerodynamics", supervisor (CharField) "Dr. Winter"> ]"""
-        assert res == res_exp
+        # https://github.com/jazzband/django-polymorphic/issues/306
+        assert {up1, up2, up3, up4, up5, up6, up7} == set(qs)
+        assert {up2, up4, up5, up6, up7} == set(UUIDArtProject.objects.all())
+        assert {up3} == set(UUIDResearchProject.objects.all())
+        assert {up4, up5, up6, up7} == set(UUIDArtProjectA.objects.all())
+        assert {up5, up6, up7} == set(UUIDArtProjectB.objects.all())
+        assert {up6, up7} == set(UUIDArtProjectC.objects.all())
+        assert {up7} == set(UUIDArtProjectD.objects.all())
 
         a = UUIDPlainA.objects.create(field1="A1")
         b = UUIDPlainB.objects.create(field1="B1", field2="B2")
@@ -1032,12 +1046,23 @@ class PolymorphicTests(TransactionTestCase):
         Model2B.objects.create(field1="A1", field2="B2")
 
         # aggregate using **kwargs
+        cnt = Count(Case(When(Model2B___field2="B2", then=1)))
         result = Model2A.objects.aggregate(
             cnt_a1=Count(Case(When(field1="A1", then=1))),
-            cnt_b2=Count(Case(When(Model2B___field2="B2", then=1))),
+            cnt_b2=cnt,
         )
         assert result == {"cnt_b2": 2, "cnt_a1": 3}
 
+        # test that our expression was immutable
+        # FIXME - expression passed into aggregate are not immutable!
+        # assert (
+        #     cnt.get_source_expressions()[0]
+        #     .get_source_expressions()[0]
+        #     .get_source_expressions()[0]
+        #     .children[0][0]
+        #     == "Model2B___field2"
+        # )
+
         # aggregate using **args
         # we have to set the defaul alias or django won't except a complex expression
         # on aggregate/annotate
@@ -1616,3 +1641,291 @@ class PolymorphicTests(TransactionTestCase):
             2,
             {"auth.User": 2},
         )
+
+    def test_manager_override(self):
+        from polymorphic.tests.models import MyBaseModel, MyChild1Model, MyChild2Model
+
+        child1 = MyChild1Model.objects.create(fieldA=4)
+        child2 = MyChild2Model.objects.create(fieldB=6)
+
+        assert MyBaseModel.objects.filter_by_user(5).count() == 2
+        assert child1 in MyBaseModel.objects.all()
+        assert child2 in MyBaseModel.objects.all()
+        assert MyChild1Model.objects.filter_by_user(5).count() == 1
+        assert MyChild2Model.objects.filter_by_user(5).count() == 1
+        assert MyChild1Model.objects.filter_by_user(5).count() == 1
+        assert MyChild1Model.objects.filter_by_user(5).first() == child1
+        assert MyChild2Model.objects.filter_by_user(5).first() == child2
+
+        assert MyChild2Model._default_manager is MyChild2Model.objects
+        MyChild2Model.objects.filter_by_user(6).count() == 0
+        MyChild2Model.base_manager.filter_by_user(6).count() == 1
+
+    def test_abstract_managers(self):
+        from django.db.models import Manager
+        from polymorphic.tests.models import (
+            AbstractManagerTest,
+            DerivedManagerTest,
+            DerivedManagerTest2,
+            SpecialPolymorphicManager,
+            SpecialQuerySet,
+            RelatedManagerTest,
+        )
+
+        with self.assertRaises(AttributeError):
+            AbstractManagerTest.objects
+        with self.assertRaises(AttributeError):
+            AbstractManagerTest.basic_manager
+        with self.assertRaises(AttributeError):
+            AbstractManagerTest.default_manager
+
+        assert type(DerivedManagerTest.objects) is SpecialPolymorphicManager
+        assert type(DerivedManagerTest.basic_manager) is Manager
+        assert type(DerivedManagerTest.default_manager) is PolymorphicManager
+        assert type(DerivedManagerTest._default_manager) is SpecialPolymorphicManager
+
+        assert type(DerivedManagerTest2.objects) is PolymorphicManager
+        assert type(DerivedManagerTest2.basic_manager) is Manager
+        assert type(DerivedManagerTest2.default_manager) is PolymorphicManager
+        assert type(DerivedManagerTest2._default_manager) is PolymorphicManager
+
+        dmt1 = DerivedManagerTest.objects.create(abstract_field="dmt1")
+        dmt2 = DerivedManagerTest2.objects.create(abstract_field="dmt2")
+
+        assert DerivedManagerTest.objects.has_text("dmt").count() == 2
+        assert dmt1 in DerivedManagerTest.objects.has_text("dmt")
+        assert dmt2 in DerivedManagerTest.objects.has_text("dmt")
+        assert DerivedManagerTest.objects.custom_queryset().has_text("dmt").count() == 2
+
+        assert isinstance(DerivedManagerTest.objects.has_text("dmt"), SpecialQuerySet)
+
+        with self.assertRaises(AttributeError):
+            DerivedManagerTest2.objects.has_text("dmt")
+
+        related = RelatedManagerTest.objects.create()
+        assert isinstance(related.derived, SpecialPolymorphicManager)
+
+    def test_fk_polymorphism(self):
+        from polymorphic.tests.models import FKTest, FKTestChild
+
+        child = FKTestChild.objects.create()
+
+        fk_test = FKTest.objects.create(fk=child)
+
+        assert fk_test.fk is child
+
+        fk_test = FKTest.objects.get(pk=fk_test.id)
+
+        assert fk_test.fk == child
+        assert isinstance(fk_test.fk, FKTestChild)
+
+    def test_polymorphic_extension(self):
+        from polymorphic.tests.models import (
+            NormalBase,
+            NormalExtension,
+            PolyExtension,
+            PolyExtChild,
+        )
+
+        nb = NormalBase.objects.create(nb_field=5)
+        ne = NormalExtension.objects.create(nb_field=6, ne_field="normal ext")
+        poly_ext = PolyExtension.objects.create(nb_field=6, ne_field="poly ext", poly_ext_field=7)
+        child_ext = PolyExtChild.objects.create(
+            nb_field=7, ne_field="child ext", poly_ext_field=8, poly_child_field="poly child"
+        )
+        assert set(NormalBase.objects.all()) == {
+            nb,
+            NormalBase.objects.get(pk=ne.pk),
+            NormalBase.objects.get(pk=poly_ext.pk),
+            NormalBase.objects.get(pk=child_ext.pk),
+        }
+        assert set(NormalExtension.objects.all()) == {
+            NormalExtension.objects.get(pk=ne.pk),
+            NormalExtension.objects.get(pk=poly_ext.pk),
+            NormalExtension.objects.get(pk=child_ext.pk),
+        }
+        assert set(PolyExtension.objects.all()) == {poly_ext, child_ext}
+        assert set(PolyExtChild.objects.all()) == {child_ext}
+
+    def test_manytomany_without_through_field(self):
+        from polymorphic.tests.models import Lake, RedheadDuck, RubberDuck
+
+        lake = Lake.objects.create()
+        rubber = RubberDuck.objects.create(name="Rubber")
+        redhead = RedheadDuck.objects.create(name="Redheat")
+        lake.ducks.add(rubber)
+        lake.ducks.add(redhead)
+        self.assertEqual(lake.ducks.count(), 2)
+        self.assertIsInstance(lake.ducks.all()[0], RubberDuck)
+        self.assertIsInstance(lake.ducks.all()[1], RedheadDuck)
+
+    def test_manytomany_with_through_field(self):
+        from polymorphic.tests.models import LakeWithThrough, DucksLake, RedheadDuck, RubberDuck
+
+        lake = LakeWithThrough.objects.create()
+        rubber = RubberDuck.objects.create(name="Rubber")
+        redhead = RedheadDuck.objects.create(name="Redheat")
+        DucksLake.objects.create(lake=lake, duck=rubber, time="morning")
+        DucksLake.objects.create(lake=lake, duck=redhead, time="afternoon")
+        self.assertEqual(lake.ducks.count(), 2)
+        self.assertIsInstance(lake.ducks.all()[0], RubberDuck)
+        self.assertIsInstance(lake.ducks.all()[1], RedheadDuck)
+
+    def test_create_from_super(self):
+        # run create test 3 times because initial implementation
+        # would fail after first success.
+        from polymorphic.tests.models import (
+            NormalBase,
+            NormalExtension,
+            PolyExtension,
+            PolyExtChild,
+        )
+
+        nb = NormalBase.objects.create(nb_field=1)
+        ne = NormalExtension.objects.create(nb_field=2, ne_field="ne2")
+
+        with self.assertRaises(TypeError):
+            PolyExtension.objects.create_from_super(nb, poly_ext_field=3)
+
+        pe = PolyExtension.objects.create_from_super(ne, poly_ext_field=3)
+
+        ne.refresh_from_db()
+        self.assertEqual(type(ne), NormalExtension)
+        self.assertEqual(type(pe), PolyExtension)
+        self.assertEqual(pe.pk, ne.pk)
+
+        self.assertEqual(pe.nb_field, 2)
+        self.assertEqual(pe.ne_field, "ne2")
+        self.assertEqual(pe.poly_ext_field, 3)
+        pe.refresh_from_db()
+        self.assertEqual(pe.nb_field, 2)
+        self.assertEqual(pe.ne_field, "ne2")
+        self.assertEqual(pe.poly_ext_field, 3)
+
+        pc = PolyExtChild.objects.create_from_super(pe, poly_child_field="pcf6")
+
+        pe.refresh_from_db()
+        ne.refresh_from_db()
+        self.assertEqual(type(ne), NormalExtension)
+        self.assertEqual(type(pe), PolyExtension)
+        self.assertEqual(pe.pk, ne.pk)
+        self.assertEqual(pe.pk, pc.pk)
+
+        self.assertEqual(pc.nb_field, 2)
+        self.assertEqual(pc.ne_field, "ne2")
+        self.assertEqual(pc.poly_ext_field, 3)
+        pc.refresh_from_db()
+        self.assertEqual(pc.nb_field, 2)
+        self.assertEqual(pc.ne_field, "ne2")
+        self.assertEqual(pc.poly_ext_field, 3)
+        self.assertEqual(pc.poly_child_field, "pcf6")
+
+        self.assertEqual(pe.polymorphic_ctype, ContentType.objects.get_for_model(PolyExtChild))
+        self.assertEqual(pc.polymorphic_ctype, ContentType.objects.get_for_model(PolyExtChild))
+
+        self.assertEqual(set(PolyExtension.objects.all()), {pc})
+
+        a1 = Model2A.objects.create(field1="A1a")
+        a2 = Model2A.objects.create(field1="A1b")
+
+        b1 = Model2B.objects.create(field1="B1a", field2="B2a")
+        b2 = Model2B.objects.create(field1="B1b", field2="B2b")
+
+        c1 = Model2C.objects.create(field1="C1a", field2="C2a", field3="C3a")
+        c2 = Model2C.objects.create(field1="C1b", field2="C2b", field3="C3b")
+
+        d1 = Model2D.objects.create(field1="D1a", field2="D2a", field3="D3a", field4="D4a")
+        d2 = Model2D.objects.create(field1="D1b", field2="D2b", field3="D3b", field4="D4b")
+
+        with self.assertRaises(TypeError):
+            Model2D.objects.create_from_super(b1, field3="D3x", field4="D4x")
+
+        b1_of_c = Model2B.objects.non_polymorphic().get(pk=c1.pk)
+        with self.assertRaises(TypeError):
+            Model2C.objects.create_from_super(b1_of_c, field3="C3x")
+
+        self.assertEqual(c1.polymorphic_ctype, ContentType.objects.get_for_model(Model2C))
+        dfs1 = Model2D.objects.create_from_super(b1_of_c, field4="D4x")
+        self.assertEqual(type(dfs1), Model2D)
+        self.assertEqual(dfs1.pk, c1.pk)
+        self.assertEqual(dfs1.field1, "C1a")
+        self.assertEqual(dfs1.field2, "C2a")
+        self.assertEqual(dfs1.field3, "C3a")
+        self.assertEqual(dfs1.field4, "D4x")
+        self.assertEqual(dfs1.polymorphic_ctype, ContentType.objects.get_for_model(Model2D))
+        c1.refresh_from_db()
+        self.assertEqual(c1.polymorphic_ctype, ContentType.objects.get_for_model(Model2D))
+
+        self.assertEqual(b2.polymorphic_ctype, ContentType.objects.get_for_model(Model2B))
+        cfs1 = Model2C.objects.create_from_super(b2, field3="C3y")
+        self.assertEqual(type(cfs1), Model2C)
+        self.assertEqual(cfs1.pk, b2.pk)
+        self.assertEqual(cfs1.field1, "B1b")
+        self.assertEqual(cfs1.field2, "B2b")
+        self.assertEqual(cfs1.field3, "C3y")
+        b2.refresh_from_db()
+        self.assertEqual(b2.polymorphic_ctype, ContentType.objects.get_for_model(Model2C))
+        self.assertEqual(cfs1.polymorphic_ctype, ContentType.objects.get_for_model(Model2C))
+
+        self.assertEqual(set(Model2A.objects.all()), {a1, a2, b1, dfs1, cfs1, c2, d1, d2})
+
+    def test_through_models_creates_and_reads(self):
+        from polymorphic.tests.models import (
+            BetMultiple,
+            ChoiceAthlete,
+            ChoiceBlank,
+            RankedAthlete,
+        )
+
+        bet = BetMultiple.objects.create()
+
+        a1 = ChoiceAthlete.objects.create(choice="Alice")
+        a2 = ChoiceAthlete.objects.create(choice="Bob")
+        a3 = ChoiceBlank.objects.create()
+
+        # Exercise the "through" model via the M2M manager using through_defaults.
+        bet.answer.add(a1, through_defaults={"rank": 2})
+        bet.answer.add(a2, through_defaults={"rank": 1})
+        bet.answer.add(a3, through_defaults={"rank": 3})  # ChoiceBlank also works
+
+        # Through rows were created with rank preserved
+        rows = list(
+            RankedAthlete.objects.filter(bet=bet)
+            .order_by("rank")
+            .values_list("choiceAthlete_id", "rank")
+        )
+        assert rows == [(a2.pk, 1), (a1.pk, 2), (a3.pk, 3)]
+
+        # Reading back via the M2M returns polymorphic instances (ChoiceAthlete, not ChoiceBlank)
+        answers = list(bet.answer.order_by("rankedathlete__rank"))
+        assert answers == [a2, a1, a3]
+        assert isinstance(answers[0], ChoiceAthlete)
+        assert isinstance(answers[1], ChoiceAthlete)
+        assert isinstance(answers[2], ChoiceBlank)
+
+        # Sanity: the through model is the one we expect
+        assert bet.answer.through is RankedAthlete
+        assert isinstance(bet.answer, PolymorphicManager)
+
+    def test_through_model_updates(self):
+        from polymorphic.tests.models import BetMultiple, ChoiceAthlete, RankedAthlete, ChoiceBlank
+
+        bet = BetMultiple.objects.create()
+        a1 = ChoiceAthlete.objects.create(choice="Alice")
+        a2 = ChoiceBlank.objects.create()
+
+        bet.answer.add(a2, through_defaults={"rank": 0})
+        bet.answer.add(a1, through_defaults={"rank": 1})
+        ra = RankedAthlete.objects.get(bet=bet, choiceAthlete=a1)
+        assert ra.rank == 1
+
+        ra.rank = 99
+        ra.save(update_fields=["rank"])
+
+        ra2 = RankedAthlete.objects.get(bet=bet, choiceAthlete=a2)
+        assert ra2.rank == 0
+
+        # ordering uses the through-table rank
+        assert list(bet.answer.order_by("rankedathlete__rank")) == [a2, a1]
+        assert RankedAthlete.objects.get(pk=ra.pk).rank == 99
+        assert RankedAthlete.objects.get(pk=ra2.pk).rank == 0
